@@ -2,10 +2,11 @@
 Basic items.
 """
 
-__version__ = "$Revision: 1579 $"
-# $HeadURL: https://svn.devjavu.com/gaphor/gaphas/tags/gaphas-0.2.0/gaphas/item.py $
+__version__ = "$Revision: 1917 $"
+# $HeadURL: https://svn.devjavu.com/gaphor/gaphas/trunk/gaphas/item.py $
 
 from math import atan2
+from weakref import WeakKeyDictionary
 
 from matrix import Matrix
 from geometry import distance_line_point, distance_rectangle_point
@@ -116,15 +117,39 @@ class Item(object):
     Base class (or interface) for items on a canvas.Canvas.
 
     Attributes:
-     - _canvas:  canvas, which owns an item
-     - _handles: list of handles owned by an item
-     - _matrix:  item's transformation matrix
+
+    - matrix: item's transformation matrix
+    - canvas: canvas, which owns an item
+    
+    Private:
+
+    - _canvas:      canvas, which owns an item
+    - _handles:     list of handles owned by an item
+    - _constraints: item's constraints
+    - _matrix_i2c:  item to canvas coordinates matrix
+    - _matrix_c2i:  canvas to item coordinates matrix
+    - _matrix_i2v:  item to view coordinates matrices
+    - _matrix_v2i:  view to item coordinates matrices
+    - _sort_key:  used to sort items
     """
 
     def __init__(self):
         self._canvas = None
         self._matrix = Matrix()
         self._handles = []
+        self._constraints = []
+
+        # used by gaphas.canvas.Canvas to hold conversion matrices
+        self._matrix_i2c = None
+        self._matrix_2ci = None
+
+        # used by gaphas.view.GtkView to hold item 2 view matrices (view=key)
+        self._matrix_i2v = WeakKeyDictionary()
+        self._matrix_v2i = WeakKeyDictionary()
+
+        # Used by gaphas.sorter.Sorter to order items fast.
+        self._sort_key = None
+
 
     @observed
     def _set_canvas(self, canvas):
@@ -148,12 +173,16 @@ class Item(object):
                 doc="Set canvas for the item. Application should use " + \
                     "Canvas.add() and Canvas.remove().")
 
+
     def setup_canvas(self):
         """
         Called when the canvas is set for the item.
         This method can be used to create constraints.
         """
-        pass
+        add = self.canvas.solver.add_constraint
+        for c in self._constraints:
+            add(c)
+
 
     def teardown_canvas(self):
         """
@@ -163,6 +192,11 @@ class Item(object):
         for h in self.handles():
             h.disconnect()
 
+        remove = self.canvas.solver.remove_constraint
+        for c in self._constraints:
+            remove(c)
+
+
     @observed
     def _set_matrix(self, matrix):
         """
@@ -174,25 +208,36 @@ class Item(object):
 
     matrix = reversible_property(lambda s: s._matrix, _set_matrix)
 
-    def request_update(self):
+    def request_update(self, update=True, matrix=True):
         if self._canvas:
-            self._canvas.request_update(self)
+            self._canvas.request_update(self, update=update, matrix=matrix)
+
 
     def pre_update(self, context):
         """
-        Do small things that have to be done before the "real" update.
-        Context has the following attributes:
+        Perform any changes before item update here, for example:
 
-        - canvas: the owning canvas
-        - matrix_i2w: Item to World transformation matrix
-        - ... (do I need something for text processing?)
+        - change matrix
+        - move handles
+
+        Gaphas does not guarantee that any canvas invariant is valid at
+        this point (i.e. constraints are not solved, first handle is not in
+        position (0, 0), etc).
         """
         pass
 
 
-    def update(self, context):
+    def post_update(self, context):
         """
-        Like pre_update(), but this is step 2.
+        Method called after item update.
+
+        If some variables should be used during drawing or in another
+        update, then they should be calculated in post method.
+
+        Changing matrix or moving handles programmatically is really not
+        advised to be performed here.
+
+        All canvas invariants are true.
         """
         pass
 
@@ -201,10 +246,10 @@ class Item(object):
         Render the item to a canvas view.
         Context contains the following attributes:
 
-        - matrix_i2w: Item to World transformation matrix (no need to)
         - cairo: the Cairo Context use this one to draw
         - view: the view that is to be rendered to
-        - selected, focused, hovered, dropzone: view state of items
+        - selected, focused, hovered, dropzone: view state of items (True/False)
+        - draw_all: a request to draw everything, for bounding box calculations
         """
         pass
 
@@ -216,8 +261,8 @@ class Item(object):
 
     def point(self, x, y):
         """
-        Get the distance from a point (@x, @y) to the item.
-        @x and @y are in item coordinates.
+        Get the distance from a point (``x``, ``y``) to the item.
+        ``x`` and ``y`` are in item coordinates.
         """
         pass
 
@@ -229,20 +274,52 @@ class Item(object):
 
 class Element(Item):
     """
-    An Element has 4 handles (for a start):
+    An Element has 4 handles (for a start)::
+
      NW +---+ NE
+        |   |
      SW +---+ SE
     """
 
     def __init__(self, width=10, height=10):
         super(Element, self).__init__()
         self._handles = [ h(strength=VERY_STRONG) for h in [Handle]*4 ]
-        self._constraints = []
-        self._min_width = 10
-        self._min_height = 10
+
+        eq = EqualsConstraint
+        lt = LessThanConstraint
+        handles = self._handles
+        h_nw = handles[NW]
+        h_ne = handles[NE]
+        h_sw = handles[SW]
+        h_se = handles[SE]
+
+        # create minimal size constraints
+        self._c_min_w = LessThanConstraint(smaller=h_nw.x, bigger=h_se.x, delta=10)
+        self._c_min_h = LessThanConstraint(smaller=h_nw.y, bigger=h_se.y, delta=10)
+
+        # setup constraints
+        self._constraints.extend([
+            eq(a=h_nw.y, b=h_ne.y),
+            eq(a=h_nw.x, b=h_sw.x),
+            eq(a=h_se.y, b=h_sw.y),
+            eq(a=h_se.x, b=h_ne.x),
+            # set h_nw < h_se constraints
+            # with minimal size functionality
+            self._c_min_w,
+            self._c_min_h,
+        ])
+
+        # set width/height when minimal size constraints exist
         self.width = width
         self.height = height
 
+    def setup_canvas(self):
+        super(Element, self).setup_canvas()
+
+        # Set width/height explicitly, so the element will maintain it
+        self.width = self.width
+        self.height = self.height
+
     def _set_width(self, width):
         """
         >>> b=Element()
@@ -259,6 +336,7 @@ class Element(Item):
         h = self._handles
         h[SE].x = h[NW].x + width
 
+
     def _get_width(self):
         """
         Width of the box, calculated as the distance from the left and
@@ -300,124 +378,32 @@ class Element(Item):
     @observed
     def _set_min_width(self, min_width):
         """
+        Set minimal width.
         """
-        self._min_width = max(0, min_width)
-        if min_width > self.width:
-            self.width = min_width
+        if min_width < 0:
+            raise ValueError, 'Minimal width cannot be less than 0'
 
-    min_width = reversible_property(lambda s: s._min_width, _set_min_width)
+        self._c_min_w.delta = min_width
+        if self.canvas:
+            self.canvas.solver.request_resolve_constraint(self._c_min_w)
+
+    min_width = reversible_property(lambda s: s._c_min_w.delta, _set_min_width)
 
     @observed
     def _set_min_height(self, min_height):
         """
+        Set minimal height.
         """
-        self._min_height = max(0, min_height)
-        if min_height > self.height:
-            self.height = min_height
+        if min_height < 0:
+            raise ValueError, 'Minimal height cannot be less than 0'
 
-    min_height = reversible_property(lambda s: s._min_height, _set_min_height)
+        self._c_min_h.delta = min_height
+        if self.canvas:
+            self.canvas.solver.request_resolve_constraint(self._c_min_h)
 
-    def setup_canvas(self):
-        """
-        >>> from canvas import Canvas
-        >>> c=Canvas()
-        >>> c.solver._constraints
-        set([])
-        >>> b = Element()
-        >>> c.add(b)
-        >>> b.canvas is c
-        True
-        >>> len(c.solver._constraints)
-        8
-        >>> len(c.solver._marked_cons)
-        0
-        >>> c.solver.solve()
-        >>> len(c.solver._constraints)
-        8
-        >>> len(c.solver._marked_cons)
-        0
-        >>> b._handles[SE].pos = (25,30)
-        >>> len(c.solver._marked_cons)
-        4
-        >>> c.solver.solve()
-        >>> float(b._handles[NE].x)
-        25.0
-        >>> float(b._handles[SW].y)
-        30.0
-        """
-        eq = EqualsConstraint
-        lt = LessThanConstraint
-        h = self._handles
-        add = self.canvas.solver.add_constraint
-        self._constraints = [
-            add(eq(a=h[NW].y, b=h[NE].y)),
-            add(eq(a=h[SW].y, b=h[SE].y)),
-            add(eq(a=h[NW].x, b=h[SW].x)),
-            add(eq(a=h[NE].x, b=h[SE].x)),
-            add(lt(smaller=h[NW].x, bigger=h[NE].x)),
-            add(lt(smaller=h[SW].x, bigger=h[SE].x)),
-            add(lt(smaller=h[NE].y, bigger=h[SE].y)),
-            add(lt(smaller=h[NW].y, bigger=h[SW].y)),
-            ]
+    min_height = reversible_property(lambda s: s._c_min_h.delta, _set_min_height)
 
-        # Immediately solve the constraints, ensuring the box is drawn okay
-        solve_for = (h[NE].y, h[SW].y, h[SW].x, h[NE].x)
-        for c, v in zip(self._constraints, solve_for):
-            c.solve_for(v)
         
-    def teardown_canvas(self):
-        """
-        Remove constraints created in setup_canvas().
-        >>> from canvas import Canvas
-        >>> c=Canvas()
-        >>> c.solver._constraints
-        set([])
-        >>> b = Element()
-        >>> c.add(b)
-        >>> b.canvas is c
-        True
-        >>> len(c.solver._constraints)
-        8
-        >>> b.teardown_canvas()
-        >>> len(c.solver._constraints)
-        0
-        """
-        super(Element, self).teardown_canvas()
-        for c in self._constraints:
-            self.canvas.solver.remove_constraint(c)
-
-
-    def pre_update(self, context):
-        """
-        Make sure handles do not overlap during movement.
-        Make sure the first handle (normally NW) is located at (0, 0).
-
-        >>> from canvas import Canvas
-        >>> c = Canvas()
-        >>> e = Element()
-        >>> c.add(e)
-        >>> e.min_width = e.min_height = 0
-        >>> c.update_now()
-        >>> e._handles
-        [<Handle object on (0, 0)>, <Handle object on (10, 0)>, <Handle object on (10, 10)>, <Handle object on (0, 10)>]
-        >>> e._handles[0].x += 1
-        >>> map(float, e._handles[0].pos)
-        [1.0, 0.0]
-        >>> c.request_update(e)
-        >>> e._handles
-        [<Handle object on (0, 0)>, <Handle object on (9, 0)>, <Handle object on (9, 10)>, <Handle object on (0, 10)>]
-        """
-        if self.width < self.min_width:
-            self.width = self.min_width
-        if self.height < self.min_height:
-            self.height = self.min_height
-
-    def update(self, context):
-        """
-        Do nothing during update.
-        """
-        pass
-
     def point(self, x, y):
         """
         Distance from the point (x, y) to the item.
@@ -498,7 +484,8 @@ class Line(Item):
                 cons.append(add(eq(a=h0.x, b=h1.x)))
             else:
                 cons.append(add(eq(a=h0.y, b=h1.y)))
-            self.canvas.solver.mark_dirty(h1.x, h1.y)
+            self.canvas.solver.request_resolve(h1.x)
+            self.canvas.solver.request_resolve(h1.y)
         self.request_update()
 
     orthogonal = reversible_property(lambda s: bool(s._orthogonal), _set_orthogonal)
@@ -522,6 +509,7 @@ class Line(Item):
         """
         Setup constraints. In this case orthogonal.
         """
+        super(Line, self).setup_canvas()
         self.orthogonal = self.orthogonal
 
     def teardown_canvas(self):
@@ -533,26 +521,45 @@ class Line(Item):
             self.canvas.solver.remove_constraint(c)
 
     @observed
+    def _reversible_insert_handle(self, index, handle):
+        self._handles.insert(index, handle)
+
+    @observed
+    def _reversible_remove_handle(self, handle):
+        self._handles.remove(handle)
+
+    reversible_pair(_reversible_insert_handle, _reversible_remove_handle, \
+            bind1={'index': lambda self, handle: self._handles.index(handle)})
+
+
     def split_segment(self, segment, parts=2):
         """
-        Split one segment in the Line in @parts equal pieces.
-        @segment 0 is the first segment (between handles 0 and 1).
+        Split one segment in the Line in ``parts`` equal pieces.
+        ``segment`` 0 is the first segment (between handles 0 and 1).
         The min number of parts is 2.
 
+        A list of new handles is returned.
+
+        Note that ``split_segment`` is not able to reconnect constraints that
+        are connected to the segment. 
+
         >>> a = Line()
         >>> a.handles()[1].pos = (20, 0)
         >>> len(a.handles())
         2
         >>> a.split_segment(0)
-        >>> len(a.handles())
-        3
-        >>> a.handles()[1]
-        <Handle object on (10, 0)>
+        [<Handle object on (10, 0)>]
+        >>> a.handles()
+        [<Handle object on (0, 0)>, <Handle object on (10, 0)>, <Handle object on (20, 0)>]
+
+        A line segment can be split into multiple (equal) parts:
+
         >>> b = Line()
         >>> b.handles()[1].pos = (20, 16)
         >>> b.handles()
         [<Handle object on (0, 0)>, <Handle object on (20, 16)>]
         >>> b.split_segment(0, parts=4)
+        [<Handle object on (5, 4)>, <Handle object on (10, 8)>, <Handle object on (15, 12)>]
         >>> len(b.handles())
         5
         >>> b.handles()
@@ -565,45 +572,59 @@ class Line(Item):
             h1 = self._handles[segment + 1]
             dx, dy = h1.x - h0.x, h1.y - h0.y
             new_h = Handle(h0.x + dx / parts, h0.y + dy / parts, strength=WEAK)
-            self._handles.insert(segment + 1, new_h)
-            # TODO: reconnect connected handles.
+            self._reversible_insert_handle(segment + 1, new_h)
             if parts > 2:
                 do_split(segment + 1, parts - 1)
         do_split(segment, parts)
+        # Force orthogonal constraints to be recreated
         self.orthogonal = self.orthogonal
+        return self._handles[segment+1:segment+parts]
 
-    @observed
     def merge_segment(self, segment, parts=2):
         """
-        Merge the @segment and the next.
+        Merge the ``segment`` and the next.
         The parts parameter indicates how many segments should be merged
 
+        The deleted handles are returned as a list.
+
         >>> a = Line()
         >>> a.handles()[1].pos = (20, 0)
-        >>> a.split_segment(0)
+        >>> _ = a.split_segment(0)
         >>> a.handles()
         [<Handle object on (0, 0)>, <Handle object on (10, 0)>, <Handle object on (20, 0)>]
         >>> a.merge_segment(0)
-        >>> len(a.handles())
-        2
+        [<Handle object on (10, 0)>]
+        >>> a.handles()
+        [<Handle object on (0, 0)>, <Handle object on (20, 0)>]
         >>> try: a.merge_segment(0)
         ... except AssertionError: print 'okay'
         okay
+
+        More than two segments can be merged at once:
+        >>> _ = a.split_segment(0)
+        >>> _ = a.split_segment(0)
+        >>> _ = a.split_segment(0)
+        >>> a.handles()
+        [<Handle object on (0, 0)>, <Handle object on (2.5, 0)>, <Handle object on (5, 0)>, <Handle object on (10, 0)>, <Handle object on (20, 0)>]
+        >>> a.merge_segment(0, parts=4)
+        [<Handle object on (2.5, 0)>, <Handle object on (5, 0)>, <Handle object on (10, 0)>]
+        >>> a.handles()
+        [<Handle object on (0, 0)>, <Handle object on (20, 0)>]
         """
         assert len(self._handles) > 2, 'Not enough segments'
         if 0 >= segment > len(self._handles) - 1:
             raise IndexError("index out of range (0 > %d > %d)" % (segment, len(self._handles) - 1))
         if segment == 0: segment = 1
-        del self._handles[segment]
+        deleted_handles = self._handles[segment:segment+parts-1]
+        self._reversible_remove_handle(self._handles[segment])
         if parts > 2:
-            merge_segment(segment, parts - 1)
+            self.merge_segment(segment, parts - 1)
         else:
             # Force orthogonal constraints to be recreated
             self.orthogonal = self.orthogonal
+        return deleted_handles
 
-    reversible_pair(split_segment, merge_segment)
 
-    
     def opposite(self, handle):
         """
         Given the handle of one end of the line, return the other end.
@@ -616,10 +637,10 @@ class Line(Item):
         else:
             raise KeyError('Handle is not an end handle')
 
-    def update(self, context):
+    def post_update(self, context):
         """
         """
-        super(Line, self).update(context)
+        super(Line, self).post_update(context)
         h0, h1 = self._handles[:2]
         self._head_angle = atan2(h1.y - h0.y, h1.x - h0.x)
         h1, h0 = self._handles[-2:]
@@ -648,6 +669,7 @@ class Line(Item):
         >>> a = Line()
         >>> a.handles()[1].pos = 30, 30
         >>> a.split_segment(0)
+        [<Handle object on (15, 15)>]
         >>> a.handles()[1].pos = 25, 5
         >>> a.point(-1, 0)
         1.0
@@ -687,16 +709,18 @@ class Line(Item):
                 draw(context)
             finally:
                 cr.restore()
+
         cr = context.cairo
         cr.set_line_width(self.line_width)
         draw_line_end(self._handles[0], self._head_angle, self.draw_head)
         for h in self._handles[1:-1]:
-            cr.line_to(float(h.x), float(h.y))
+            cr.line_to(h.x, h.y)
         h0, h1 = self._handles[-2:]
         draw_line_end(self._handles[-1], self._tail_angle, self.draw_tail)
         cr.stroke()
 
 
+
 __test__ = {
     'Line._set_orthogonal': Line._set_orthogonal,
     'Line._set_horizontal': Line._set_horizontal,
