@@ -7,8 +7,8 @@ Each painter takes are of a layer in the canvas (such as grid, items
 and handles).
 """
 
-__version__ = "$Revision: 1580 $"
-# $HeadURL: https://svn.devjavu.com/gaphor/gaphas/tags/gaphas-0.2.0/gaphas/painter.py $
+__version__ = "$Revision: 1715 $"
+# $HeadURL: https://svn.devjavu.com/gaphor/gaphas/trunk/gaphas/painter.py $
 
 from cairo import Matrix, ANTIALIAS_NONE, LINE_JOIN_ROUND
 
@@ -69,9 +69,8 @@ class DrawContext(Context):
     focused.
     """
 
-    painter = None
-    cairo = None
-
+    deprecated = False
+    
     def __init__(self, **kwargs):
         super(DrawContext, self).__init__(**kwargs)
 
@@ -80,10 +79,14 @@ class DrawContext(Context):
         Extra helper method for drawing child items from within
         the Item.draw() method.
         """
-        self.painter._draw_items(self._item.canvas.get_children(self._item),
-                                 self.view,
-                                 self.cairo,
-                                 self._area)
+        if not DrawContext.deprecated:
+            print 'Usage of context.draw_children is deprecated.'
+            DrawContext.deprecated = True
+
+        #self.painter._draw_items(self._item.canvas.get_children(self._item),
+        #                         self.view,
+        #                         self.cairo,
+        #                         self._area)
 
 
 class ItemPainter(Painter):
@@ -94,7 +97,7 @@ class ItemPainter(Painter):
         cairo.save()
         try:
             cairo.set_matrix(view.matrix)
-            cairo.transform(view.canvas.get_matrix_i2w(item))
+            cairo.transform(view.canvas.get_matrix_i2c(item))
 
             item.draw(DrawContext(painter=self,
                                   view=view,
@@ -116,10 +119,10 @@ class ItemPainter(Painter):
         to draw sub-items.
         """
         for item in items:
-            if not area or view.get_item_bounding_box(item) - area:
-                self._draw_item(item, view, cairo, area=area)
-                if DEBUG_DRAW_BOUNDING_BOX:
-                    self._draw_bounds(item, view, cairo)
+            #if not area or area - view.get_item_bounding_box(item):
+            self._draw_item(item, view, cairo, area=area)
+            if DEBUG_DRAW_BOUNDING_BOX:
+                self._draw_bounds(item, view, cairo)
 
     def _draw_bounds(self, item, view, cairo):
         try:
@@ -139,7 +142,7 @@ class ItemPainter(Painter):
         cairo = context.cairo
         view = context.view
         area = context.area
-        items = view.canvas.get_root_items()
+        items = context.items
         cairo.set_tolerance(TOLERANCE)
         cairo.set_line_join(LINE_JOIN_ROUND)
         self._draw_items(items, view, cairo, area)
@@ -253,14 +256,15 @@ class BoundingBoxPainter(ItemPainter):
         bounds = cairo.get_bounds()
 
         # Update bounding box with handles.
-        transform_i2c = (view.canvas.get_matrix_i2w(item) * view._matrix).transform_point
+        i2v = view.get_matrix_i2v(item).transform_point
         for h in item.handles():
-            cx, cy = transform_i2c(h.x, h.y)
+            cx, cy = i2v(h.x, h.y)
             bounds += (cx - 5, cy - 5, 9, 9)
 
         bounds.expand(1)
         view.set_item_bounding_box(item, bounds)
 
+
     def _draw_items(self, items, view, cairo, area=None):
         """
         Draw the items. This method can also be called from DrawContext
@@ -287,8 +291,7 @@ class HandlePainter(Painter):
         The handles are drawn in non-antialiased mode for clearity.
         """
         cairo.save()
-        m = Matrix(*view.canvas.get_matrix_i2w(item))
-        m *= view._matrix
+        i2v = view.get_matrix_i2v(item)
         if not opacity:
             opacity = (item is view.focused_item) and .7 or .4
 
@@ -306,7 +309,7 @@ class HandlePainter(Painter):
 
             cairo.identity_matrix()
             cairo.set_antialias(ANTIALIAS_NONE)
-            cairo.translate(*m.transform_point(h.x, h.y))
+            cairo.translate(*i2v.transform_point(h.x, h.y))
             cairo.rectangle(-4, -4, 8, 8)
             cairo.set_source_rgba(r, g, b, opacity)
             cairo.fill_preserve()
@@ -321,11 +324,9 @@ class HandlePainter(Painter):
 
     def paint(self, context):
         view = context.view
+        canvas = view.canvas
         cairo = context.cairo
-        items = view.canvas.get_all_items()
-        # Draw handles of selected items on top of the items.
-        # Compare with canvas.get_all_items() to determine drawing order.
-        for item in (i for i in items if i in view.selected_items):
+        for item in view.selected_items:
             self._draw_handles(item, view, cairo)
         item = view.hovered_item
         if item and item not in view.selected_items:
