@@ -8,12 +8,12 @@ Variables change and at some point of time we want to make all constraints
 valid again. This process is called solving constraints.
 
 Gaphas' solver allows to define constraints between Variable instances.
-Constraint classes are defined in gaphas.constraint module.
+Constraint classes are defined in `gaphas.constraint` module.
 
 How It Works
-============
+------------
 Every constraint contains list of variables and has to be registered in
-solver object. Variables change (Variables.dirty, Solver.mark_dirty
+solver object. Variables change (`Variable.dirty()`, `Solver.request_resolve()`
 methods) and their constraints are marked by solver as dirty. To solve
 constraints, solver loops through dirty constraints and asks constraint for
 a variable (called weakest variable), which
@@ -26,19 +26,22 @@ a variable (called weakest variable), which
 
 (weakest variable invariants defined above)
 
-Having weakest variable (Constraint.weakest method) every constraint is
-being asked to solve itself (Constraint.solv_for method) changing
-appropriate variables to make the constraint valid again.
+Having weakest variable (`constraint.Constraint.weakest()` method) every
+constraint is being asked to solve itself (`constraint.Constraint.solve_for()`
+method) changing appropriate variables to make the constraint valid again.
 """
 
 from __future__ import division
 
-__version__ = "$Revision: 1542 $"
-# $HeadURL: https://svn.devjavu.com/gaphor/gaphas/tags/gaphas-0.2.0/gaphas/solver.py $
+__version__ = "$Revision: 2093 $"
+# $HeadURL: https://svn.devjavu.com/gaphor/gaphas/trunk/gaphas/solver.py $
 
 from operator import isCallable
 from state import observed, reversible_pair, reversible_property
 
+# epsilon for float comparison
+# is simple abs(x - y) > EPSILON enough for canvas needs?
+EPSILON = 1e-10
 
 # Variable Strengths:
 VERY_WEAK = 0
@@ -76,12 +79,19 @@ class Variable(object):
     strength = reversible_property(lambda s: s._strength, _set_strength)
 
     def dirty(self):
+        """
+        Mark the variable dirty in both the constraint solver and attached
+        constraints.
+        
+        Variables are marked dirty also during constraints solving to
+        solve all dependent constraints, i.e. two equals constraints
+        between 3 variables.
+        """
         solver = self._solver
-        if not solver: # or solver._solving:
+        if not solver:
             return
-        solver.mark_dirty(self)
-        for c in self._constraints:
-            c.mark_dirty(self)
+
+        solver.request_resolve(self)
 
     @observed
     def set_value(self, value):
@@ -106,7 +116,7 @@ class Variable(object):
         >>> Variable(5) != 5
         False
         """
-        return self._value.__eq__(float(other))
+        return abs(self._value - other) < EPSILON
 
     def __ne__(self, other):
         """
@@ -115,7 +125,7 @@ class Variable(object):
         >>> Variable(5) != 5
         False
         """
-        return self._value.__ne__(float(other))
+        return abs(self._value - other) > EPSILON
 
     def __gt__(self, other):
         """
@@ -221,7 +231,6 @@ class Variable(object):
 
     def __truediv__(self, other):
         """
-        #>>> from __future__ import division
         >>> Variable(5.) / 4
         1.25
         >>> 10 / Variable(5.)
@@ -291,13 +300,67 @@ class Variable(object):
 
     def __rtruediv__(self, other):
         """
-        #>>> from __future__ import division
         >>> 5. / Variable(4)
         1.25
         """
         return self._value.__rtruediv__(other)
 
 
+class Projection(object):
+    """
+    Projections are used to convert values from one space to another,
+    e.g. from Canvas to Item space or visa versa.
+
+    In order to be a Projection the ``value`` and ``strength`` properties
+    should be implemented and a method named ``variable()`` should be present.
+
+    Projections should inherit from this class.
+
+    Projections may be nested.
+
+    This default implementation projects a variable to it's own:
+
+    >>> v = Variable(4.0)
+    >>> v
+    Variable(4, 20)
+    >>> p = Projection(v)
+    >>> p.value
+    4.0
+    >>> p.value = -1
+    >>> p.value
+    -1.0
+    >>> v.value
+    -1.0
+    >>> p.strength
+    20
+    >>> p.variable()
+    Variable(-1, 20)
+    """
+
+    def __init__(self, var):
+        self._var = var
+
+    def _set_value(self, value):
+        self._var.value = value
+
+    value = property(lambda s: s._var.value, _set_value)
+
+    strength = property(lambda s: s.variable().strength)
+
+    def variable(self):
+        """
+        Return the variable owned by the projection.
+        """
+        return self._var
+
+    def __float__(self):
+        return float(self.variable()._value)
+
+    def __str__(self):
+        return '%s(%s)' % (self.__class__.__name__, self.variable())
+    __repr__ = __str__
+
+
 class Solver(object):
     """
     Solve constraints. A constraint should have accompanying
@@ -310,12 +373,17 @@ class Solver(object):
         self._marked_cons = []
         self._solving = False
 
-    def mark_dirty(self, *variables):
+    def request_resolve(self, variable, projections_only=False):
         """
         Mark a variable as "dirty". This means it it solved the next time
         the constraints are resolved.
 
+        If projections_only is set to True, only constraints using the
+        variable through a Projection instance (e.i. variable itself is not
+        in `constraint.Constraint.variables()`) are marked.
+
         Example:
+
         >>> from constraint import EquationConstraint
         >>> a, b, c = Variable(1.0), Variable(2.0), Variable(3.0)
         >>> s = Solver()
@@ -336,23 +404,20 @@ class Solver(object):
         >>> c_eq.weakest()
         Variable(2, 20)
         """
-        for variable in variables:
-            for c in variable._constraints:
+        for c in variable._constraints:
+            if not projections_only or c._solver_has_projections:
                 if not self._solving:
                     if c in self._marked_cons:
                         self._marked_cons.remove(c)
+                    c.mark_dirty(variable)
                     self._marked_cons.append(c)
-                elif c not in self._marked_cons:
+                else:
+                    c.mark_dirty(variable)
                     self._marked_cons.append(c)
+                    if __debug__:
+                        if self._marked_cons.count(c) > 100:
+                            raise JuggleError, 'Variable juggling detected, constraint %s' % c
 
-    def request_resolve(self, variable):
-        """
-        Request resolving of the constraints for a variable. This does not
-        mark the variable itself as dirty.
-        """
-        for c in variable._constraints:
-            if c not in self._marked_cons:
-                self._marked_cons.append(c)
 
     @observed
     def add_constraint(self, constraint):
@@ -362,6 +427,7 @@ class Solver(object):
         later on.
 
         Example:
+
         >>> from constraint import EquationConstraint
         >>> s = Solver()
         >>> a, b = Variable(), Variable(2.0)
@@ -376,14 +442,14 @@ class Solver(object):
         >>> len(s._constraints)
         1
         """
-        #if isCallable(constraint):
-        #    from constraint import EquationConstraint
-        #    constraint = EquationConstraint(constraint)
-        #constraint.set(**variables)
-        #print constraint
+        assert constraint, 'No constraint (%s)' % (constraint,)
         self._constraints.add(constraint)
         self._marked_cons.append(constraint)
+        constraint._solver_has_projections = False
         for v in constraint.variables():
+            while isinstance(v, Projection):
+                v = v.variable()
+                constraint._solver_has_projections = True
             v._constraints.add(constraint)
             v._solver = self
         #print 'added constraint', constraint
@@ -391,7 +457,9 @@ class Solver(object):
 
     @observed
     def remove_constraint(self, constraint):
-        """ Remove a constraint from the solver
+        """
+        Remove a constraint from the solver
+
         >>> from constraint import EquationConstraint
         >>> s = Solver()
         >>> a, b = Variable(), Variable(2.0)
@@ -405,46 +473,103 @@ class Solver(object):
         set([])
 
         Removing a constraint twice has no effect:
-        >>> s.remove_constraint(c)
 
+        >>> s.remove_constraint(c)
         """
+        assert constraint, 'No constraint (%s)' % (constraint,)
         for v in constraint.variables():
+            while isinstance(v, Projection):
+                v = v.variable()
             v._constraints.discard(constraint)
         self._constraints.discard(constraint)
-        if constraint in self._marked_cons:
-            del self._marked_cons[self._marked_cons.index(constraint)]
+        while constraint in self._marked_cons:
+            self._marked_cons.remove(constraint)
 
     reversible_pair(add_constraint, remove_constraint)
 
-    def constraints_with_variable(self, variable):
+
+    def request_resolve_constraint(self, c):
+        """
+        Request resolving a constraint.
+        """
+        self._marked_cons.append(c)
+
+
+    def constraints_with_variable(self, *variables):
         """
         Return an iterator of constraints that work with variable.
         The variable in question should be exposed by the constraints
-        variables() method.
+        `constraint.Constraint.variables()` method.
 
         >>> from constraint import EquationConstraint
         >>> s = Solver()
-        >>> a, b = Variable(), Variable(2.0)
-        >>> s.add_constraint(EquationConstraint(lambda a, b: a -b, a=a, b=b))
+        >>> a, b, c = Variable(), Variable(2.0), Variable(4.0)
+        >>> eq_a_b = s.add_constraint(EquationConstraint(lambda a, b: a -b, a=a, b=b))
+        >>> eq_a_b
         EquationConstraint(<lambda>, a=Variable(0, 20), b=Variable(2, 20))
-        >>> s.add_constraint(EquationConstraint(lambda a, b: a -b, a=a, b=b))
-        EquationConstraint(<lambda>, a=Variable(0, 20), b=Variable(2, 20))
-        >>> len(s._constraints)
-        2
-        >>> for c in s.constraints_with_variable(a): print c
-        EquationConstraint(<lambda>, a=Variable(0, 20), b=Variable(2, 20))
-        EquationConstraint(<lambda>, a=Variable(0, 20), b=Variable(2, 20))
-        """
-        # use a copy of the original set, so constraints may be deleted in the
-        # meantime.
-        for c in set(self._constraints):
-            if variable in c.variables():
-                yield c
+        >>> eq_a_c = s.add_constraint(EquationConstraint(lambda a, b: a -b, a=a, b=c))
+        >>> eq_a_c
+        EquationConstraint(<lambda>, a=Variable(0, 20), b=Variable(4, 20))
 
+        And now for some testing:
+
+        >>> eq_a_b in s.constraints_with_variable(a)
+        True
+        >>> eq_a_c in s.constraints_with_variable(a)
+        True
+        >>> eq_a_b in s.constraints_with_variable(a, b)
+        True
+        >>> eq_a_c in s.constraints_with_variable(a, b)
+        False
+
+        Using another variable with the same value does not work:
+
+        >>> d = Variable(2.0)
+        >>> eq_a_b in s.constraints_with_variable(a, d)
+        False
+
+        This also works for projections:
+
+        >>> eq_pr_a_b = s.add_constraint(EquationConstraint(lambda a, b: a -b, a=Projection(a), b=Projection(b)))
+        >>> eq_pr_a_b   # doctest: +ELLIPSIS
+        EquationConstraint(<lambda>, a=Projection(Variable(0, 20)), b=Projection(Variable(2, 20)))
+
+        >>> eq_pr_a_b in s.constraints_with_variable(a, b)
+        True
+        >>> eq_pr_a_b in s.constraints_with_variable(a, c)
+        False
+        >>> eq_pr_a_b in s.constraints_with_variable(a, d)
+        False
+        """
+        # Use a copy of the original set, so constraints may be
+        # deleted in the meantime.
+        variables = set(variables)
+        for c in set(self._constraints):
+            if variables.issubset(set(c.variables())):
+                yield c
+            elif c._solver_has_projections:
+                found = True
+                for v in c.variables():
+                    if v in variables:
+                        continue
+                    while isinstance(v, Projection):
+                        v = v.variable()
+                        if v in variables:
+                            break
+                    else:
+                        found = False
+                    if not found:
+                        break # quit for loop, variable not in constraint
+                else:
+                    # All iteration have completed succesfully,
+                    # so all variables are in the constraint
+                    yield c
+                    
 
     def solve(self):
         """
         Example:
+
         >>> from constraint import EquationConstraint
         >>> a, b, c = Variable(1.0), Variable(2.0), Variable(3.0)
         >>> s = Solver()
@@ -472,7 +597,6 @@ class Solver(object):
         >>> c._value
         10.0
         """
-        constraints = self._constraints
         marked_cons = self._marked_cons
         try:
             self._solving = True
@@ -535,6 +659,15 @@ class solvable(object):
             v.value = value
 
 
+
+
+class JuggleError(AssertionError):
+    """
+    Variable juggling exception. Raised when constraint's variables are
+    marking each other dirty forever.
+    """
+
+
 __test__ = {
     'Solver.add_constraint': Solver.add_constraint,
     'Solver.remove_constraint': Solver.remove_constraint,
