@@ -21,8 +21,8 @@ Maybe even:
     (context.view = view; context.grab() to grab, context.ungrab() to ungrab)
 """
 
-__version__ = "$Revision: 1569 $"
-# $HeadURL: https://svn.devjavu.com/gaphor/gaphas/tags/gaphas-0.2.0/gaphas/tool.py $
+__version__ = "$Revision: 1916 $"
+# $HeadURL: https://svn.devjavu.com/gaphor/gaphas/trunk/gaphas/tool.py $
 
 import cairo
 import gtk
@@ -297,7 +297,7 @@ class ItemTool(Tool):
 
                 # Filter the items that should eventually be moved
                 get_ancestors = view.canvas.get_ancestors
-                selected_items = view.selected_items
+                selected_items = set(view.selected_items)
                 for i in selected_items:
                     # Do not move subitems of selected items
                     if not set(get_ancestors(i)).intersection(selected_items):
@@ -322,22 +322,14 @@ class ItemTool(Tool):
             view = context.view
             canvas = view.canvas
 
-            selected_items = view.selected_items
-
-            # First request redraws for all items, before enything is
-            # changed.
-            view.queue_draw_item(*self._movable_items)
-
             # Calculate the distance the item has to be moved
-            dx, dy = view.transform_distance_c2w(event.x - self.last_x,
-                                                 event.y - self.last_y)
-
-            get_matrix_w2i = canvas.get_matrix_w2i
+            dx, dy = event.x - self.last_x, event.y - self.last_y
 
             # Now do the actual moving.
             for i in self._movable_items:
                 # Move the item and schedule it for an update
-                i.matrix.translate(*get_matrix_w2i(i).transform_distance(dx, dy))
+                v2i = view.get_matrix_v2i(i)
+                i.matrix.translate(*v2i.transform_distance(dx, dy))
                 canvas.request_matrix_update(i)
 
             self.last_x, self.last_y = event.x, event.y
@@ -364,23 +356,18 @@ class HandleTool(Tool):
         self._grabbed_item = item
         self._grabbed_handle = handle
 
-        # Increment handle strength for the duration of the grab action
-        self._grabbed_handle.x.strength += 1
-        self._grabbed_handle.y.strength += 1
-
 
     def _find_handle(self, view, event, item):
         """
         Find item's handle at (event.x, event.y)
         """
-        transform_point = view.canvas.get_matrix_i2w(item).transform_point
-        e_x, e_y = view.transform_point_c2w(event.x, event.y)
-        dx, dy = view.transform_distance_c2w(6, 6)
+        i2v = view.get_matrix_i2v(item).transform_point
+        x, y = event.x, event.y
         for h in item.handles():
             if not h.movable:
                 continue
-            wx, wy = transform_point(h.x, h.y)
-            if -dx < (wx - e_x) < dx and -dy < (wy - e_y) < dy:
+            wx, wy = i2v(h.x, h.y)
+            if -6 < (wx - x) < 6 and -6 < (wy - y) < 6:
                 return h
         return None
 
@@ -403,65 +390,36 @@ class HandleTool(Tool):
                 return view.hovered_item, h
 
         # Last try all items, checking the bounding box first
-        get_children = view.canvas.get_children
-        get_item_bounding_box = view.get_item_bounding_box
         x, y = event.x, event.y
-        def find(parent):
-            found_item, found_h = None, None
-            for item in get_children(parent):
-                if (x, y) in get_item_bounding_box(item):
-                    item, h = find(item)
-                    if h: found_item, found_h = item, h
-
-            # Escape when a match is found or parent is the uber-root item
-            if found_h or parent is None:
-                return found_item, found_h
-
-            h = self._find_handle(view, event, parent)
+        items = view.get_items_in_rectangle((x - 6, y - 6, 12, 12), reverse=True)
+        
+        found_item, found_h = None, None
+        for item in items:
+            h = self._find_handle(view, event, item)
             if h:
-                return parent, h
-            return None, None
-
-        return find(None)
+                return item, h
+        return None, None
 
 
     def move(self, view, item, handle, x, y):
         """
-        Move the handle to position (x,y).
-        This version already has some special behavior implemented for
-        gaphas.item.Element. The min_width and min_height properties of
-        Element are used to restrict the handles from overlapping each other.
+        Move the handle to position ``(x,y)``. ``x`` and ``y`` are in
+        item coordnates.
         """
-        # Special behavior for Elements:
-        if isinstance(item, Element):
-            index = list(item.handles()).index(handle)
-            opposite = item.handles()[(index + 2) % 4]
-
-            if index == 0 or index == 3:
-                if opposite.x - x < item.min_width:
-                    x = opposite.x - item.min_width
-            else:
-                if x - opposite.x < item.min_width:
-                    x = opposite.x + item.min_width
-
-            if index == 0 or index == 1:
-                if opposite.y - y < item.min_height:
-                    y = opposite.y - item.min_height
-            else:
-                if y - opposite.y < item.min_height:
-                    y = opposite.y + item.min_height
-
         handle.x = x
         handle.y = y
 
-    def glue(self, view, item, handle, wx, wy):
+
+    def glue(self, view, item, handle, vx, vy):
         """
-        Find an item near @handle that @item can connect to.
+        Find an item near ``handle` that ``item`` can connect to.
+        ``vx`` and ``vy`` are the pointer (view) coordinates.
         """
 
-    def connect(self, view, item, handle, wx, wy):
+    def connect(self, view, item, handle, vx, vy):
         """
-        Find an item near @handle that @item can connect to and connect.
+        Find an item near ``handle`` that ``item`` can connect to and connect.
+        ``vx`` and ``vy`` are the pointer (view) coordinates.
         """
 
     def disconnect(self, view, item, handle):
@@ -501,15 +459,9 @@ class HandleTool(Tool):
         # queue extra redraw to make sure the item is drawn properly
         try:
             view = context.view
-            wx, wy = view.transform_point_c2w(event.x, event.y)
             if self._grabbed_handle and self._grabbed_handle.connectable:
-                self.connect(view, self._grabbed_item, self._grabbed_handle, wx, wy)
+                self.connect(view, self._grabbed_item, self._grabbed_handle, event.x, event.y)
         finally:
-            # Decrement handle strength, previously incremented on button press
-            if self._grabbed_handle:
-                self._grabbed_handle.x.strength -= 1
-                self._grabbed_handle.y.strength -= 1
-            context.view.queue_draw_item(context.view.hovered_item)
             context.ungrab()
         if self._grabbed_handle:
             self._grabbed_item.request_update()
@@ -527,22 +479,18 @@ class HandleTool(Tool):
             item = self._grabbed_item
             handle = self._grabbed_handle
 
-            # Do an explicit redraw request here, since we do not keep old
-            # positions of handles around.
-            view.queue_draw_item(item)
-
-            # Calculate the distance the item has to be moved
-            wx, wy = view.transform_point_c2w(event.x, event.y)
-            x, y = canvas.get_matrix_w2i(item).transform_point(wx, wy)
+            v2i = view.get_matrix_v2i(item)
+            x, y = v2i.transform_point(event.x, event.y)
 
             # Do the actual move:
             self.move(view, item, handle, x, y)
             
-            item.request_update()
-            #canvas.update_matrix(item)
+            # do not request matrix update as matrix recalculation will be
+            # performed due to item normalization if required
+            item.request_update(matrix=False)
             try:
                 if self._grabbed_handle.connectable:
-                    self.glue(view, item, handle, wx, wy)
+                    self.glue(view, item, handle, event.x, event.y)
             finally:
                 pass
             return True
@@ -557,40 +505,40 @@ class HandleTool(Tool):
 class RubberbandTool(Tool):
 
     def __init__(self):
-        self.rect = Rectangle()
+        self.x0, self.y0, self.x1, self.y1 = 0, 0, 0, 0
 
     def on_button_press(self, context, event):
         context.grab()
-        self.rect.x, self.rect.y = event.x, event.y
-        self.rect.x1, self.rect.y1 = event.x, event.y
+        self.x0, self.y0 = event.x, event.y
+        self.x1, self.y1 = event.x, event.y
         return True
 
     def on_button_release(self, context, event):
         context.ungrab()
         self.queue_draw(context.view)
-        r = self.rect
-        context.view.select_in_rectangle(r)
+        x0, y0, x1, y1 = self.x0, self.y0, self.x1, self.y1
+        context.view.select_in_rectangle((min(x0, x1), min(y0, y1),
+                 abs(x1 - x0), abs(y1 - y0)))
         return True
 
     def on_motion_notify(self, context, event):
         if event.state & gtk.gdk.BUTTON_PRESS_MASK:
             view = context.view
             self.queue_draw(view)
-            # TODO: keep track of what's expanding (top/bottom, left/right)
-            self.rect.x1, self.rect.y1 = event.x, event.y
+            self.x1, self.y1 = event.x, event.y
             self.queue_draw(view)
             return True
 
     def queue_draw(self, view):
-        r = self.rect
-        view.queue_draw_area(*r)
+        x0, y0, x1, y1 = self.x0, self.y0, self.x1, self.y1
+        view.queue_draw_area(min(x0, x1), min(y0, y1), abs(x1 - x0), abs(y1 - y0))
 
     def draw(self, context):
         cr = context.cairo
-        r = self.rect
+        x0, y0, x1, y1 = self.x0, self.y0, self.x1, self.y1
         cr.set_line_width(1.0)
         cr.set_source_rgba(.5, .5, .7, .6)
-        cr.rectangle(*r)
+        cr.rectangle(min(x0, x1), min(y0, y1), abs(x1 - x0), abs(y1 - y0))
         cr.fill()
 
 
@@ -611,8 +559,6 @@ class PlacementTool(Tool):
         view = context.view
         canvas = view.canvas
         new_item = self._create_item(context, event.x, event.y)
-        if new_item not in canvas.get_all_items():
-            canvas.add(new_item)
         self._handle_tool.grab_handle(new_item,
                                       new_item.handles()[self._handle_index])
         self._new_item = new_item
@@ -622,8 +568,9 @@ class PlacementTool(Tool):
 
     def _create_item(self, context, x, y):
         view = context.view
+        canvas = view.canvas
         item = self._factory()
-        x, y = view.transform_point_c2w(x, y)
+        x, y = view.get_matrix_v2i(item).transform_point(x, y)
         item.matrix.translate(x, y)
         return item
 
